/*
 * This program tests the operation of the radio driver.
 * @author Ben L. Titzer
 * @author Akop Palyan
 */
program TestRadio {
    entrypoint main = TestRadio.start;
    entrypoint timer0_comp = Timer0.compareHandler;
    entrypoint usart0_rx = USART0.rec_handler;
    entrypoint usart0_udre = USART0.tran_handler;
    entrypoint spi_stc = SPI.interruptHandler;
}

component TestRadio {

    field buffer: char[] = new char[16];

    field packet_char1: char[];
    field packet_char2: char[];

    field send: int;

    constructor() {
        local packet1 = { 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 
                          0x33, 0xCC, 0xFF, 0xFF, 0x04, 0x7D, 0x03, 0x04, 
                          0x00, 0x00, 0x7F, 0x4C, 0x4C, 0x4C };

        local packet2 = { 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA,
                          0x33, 0xCC, 0xFF, 0xFF, 0x04, 0x7D, 0x03, 0x05, 
                          0x00, 0x00, 0x4F, 0x7B, 0x7B, 0x7B };

        packet_char1 = convert(packet1);
        packet_char2 = convert(packet2);
    }

    method start() {
        local i: int;

        Mica2.startTimer();
        Mica2.startTerminal();
	Mica2.startSPI(false);
	CC1000.init(CC1000.PRESET0);
	CC1000.start();
	CC1000.rxMode();
        Mica2.setTimerCompare(timerInterrupt);
	MCU.enable();
        while ( true ) {
            if ( send == 1 ) {
               sendPacket(packet_char1);
               send = 2;
            }

            if ( send == 3 ) {
               sendPacket(packet_char2);
               send = 0;
            }
            
            // do nothing.
            // recvPacket();
            MCU.sleep();
	}
    }

    method recvPacket() {
        local i = 0;
	local len = SPI.read(buffer, 0, buffer.length);
        while (i < len) {
	    Terminal.printHex(buffer[i++]);
            Terminal.nextLine();
	}
    }

    method timerInterrupt() {
        if ( send == 0 ) send = 1;
        if ( send == 2 ) send = 3;
    }

    method sendPacket(packet: char[]) {
        local pos: int;
        CC1000.txMode();
        for ( pos = 0; pos < packet.length; ) {
           pos = pos + SPI.write(packet, pos, packet.length - pos);
        }
        while ( SPI.isTransmitting() ) {
           MCU.sleep(); // wait for queue to drain
        }
        CC1000.rxMode();
    }

    method convert(a: 8[]): char[] {
       local r = new char[a.length];
       local i = 0;
       for ( ; i < a.length; i++ ) r[i] = a[i] :: char;
       return r;
    }
}
